/**
 * 集成USDT转账监听与Split自动化购买
 * 实现用户支付完成后自动在Split平台购买Telegram Premium会员
 */

const { SplitAutomation } = require('./src/split-automation');
const { UsdtMonitor } = require('./src/usdt-monitor'); // 假设这是您现有的USDT监听模块
const fs = require('fs');
const path = require('path');

// 配置
const config = {
  // Split自动化配置
  split: {
    walletPath: process.env.TON_WALLET_PATH || '/path/to/ton/wallet/extension',
    credentials: {
      walletSeed: process.env.WALLET_SEED || 'your-wallet-seed-phrase',
      password: process.env.WALLET_PASSWORD || 'your-wallet-password'
    },
    headless: process.env.HEADLESS === 'true',
    timeout: parseInt(process.env.TIMEOUT || '30000'),
    logPath: process.env.LOG_PATH || path.join(__dirname, 'logs')
  },
  
  // USDT监听配置
  usdt: {
    walletAddress: process.env.WALLET_ADDRESS || 'TRmvaTGuK7SsHX72xFZcekcWBxNoRv8888',
    contractAddress: process.env.CONTRACT_ADDRESS || 'TR7NHqjeKQxGTCi8q8ZY4pL8otSzgjLj6t',
    apiKey: process.env.TRONGRID_API_KEY || 'your-trongrid-api-key',
    checkInterval: parseInt(process.env.CHECK_INTERVAL || '60000') // 默认每分钟检查一次
  },
  
  // 数据库配置（用于存储订单和交易记录）
  database: {
    path: process.env.DB_PATH || path.join(__dirname, 'data', 'transactions.json')
  }
};

// 确保数据目录存在
const dataDir = path.dirname(config.database.path);
if (!fs.existsSync(dataDir)) {
  fs.mkdirSync(dataDir, { recursive: true });
}

// 如果数据库文件不存在，创建一个空的数据库
if (!fs.existsSync(config.database.path)) {
  fs.writeFileSync(config.database.path, JSON.stringify({ transactions: [] }));
}

/**
 * 简单的数据库操作
 */
const db = {
  /**
   * 读取所有交易
   */
  getTransactions() {
    const data = JSON.parse(fs.readFileSync(config.database.path, 'utf8'));
    return data.transactions || [];
  },
  
  /**
   * 保存交易
   * @param {Object} transaction 交易数据
   */
  saveTransaction(transaction) {
    const data = JSON.parse(fs.readFileSync(config.database.path, 'utf8'));
    data.transactions = data.transactions || [];
    data.transactions.push({
      ...transaction,
      timestamp: new Date().toISOString()
    });
    fs.writeFileSync(config.database.path, JSON.stringify(data, null, 2));
    return transaction;
  },
  
  /**
   * 更新交易状态
   * @param {string} txHash 交易哈希
   * @param {string} status 新状态
   * @param {Object} additionalData 额外数据
   */
  updateTransactionStatus(txHash, status, additionalData = {}) {
    const data = JSON.parse(fs.readFileSync(config.database.path, 'utf8'));
    data.transactions = data.transactions || [];
    
    const transaction = data.transactions.find(tx => tx.txHash === txHash);
    if (transaction) {
      transaction.status = status;
      transaction.updatedAt = new Date().toISOString();
      Object.assign(transaction, additionalData);
      fs.writeFileSync(config.database.path, JSON.stringify(data, null, 2));
    }
    return transaction;
  }
};

/**
 * 日志记录
 * @param {string} message 日志消息
 * @param {string} level 日志级别
 */
function log(message, level = 'INFO') {
  const timestamp = new Date().toISOString();
  const logMessage = `[${timestamp}] [${level}] ${message}`;
  
  console.log(logMessage);
  
  const logFile = path.join(config.split.logPath, `integration-${new Date().toLocaleDateString().replace(/\//g, '-')}.log`);
  fs.appendFileSync(logFile, logMessage + '\n');
}

/**
 * 处理新的USDT转账
 * @param {Object} payment 支付信息
 */
async function handleNewPayment(payment) {
  log(`检测到新的USDT转账: ${JSON.stringify(payment)}`);
  
  try {
    // 保存交易记录
    const transaction = db.saveTransaction({
      txHash: payment.txHash,
      from: payment.from,
      to: payment.to,
      amount: payment.amount,
      telegramId: payment.telegramId,
      months: payment.months || 3, // 默认3个月
      status: 'pending',
      createdAt: new Date().toISOString()
    });
    
    log(`已保存交易记录，准备在Split平台购买Premium会员`);
    
    // 初始化Split自动化实例
    const splitBot = new SplitAutomation(config.split);
    
    try {
      // 初始化浏览器和钱包
      await splitBot.initialize();
      
      // 购买Premium会员
      const result = await splitBot.buyPremium(payment.telegramId, payment.months || 3);
      
      // 更新交易状态为成功
      db.updateTransactionStatus(payment.txHash, 'completed', {
        splitTransactionId: result.transactionId,
        completedAt: new Date().toISOString()
      });
      
      log(`成功为用户 ${payment.telegramId} 购买了 ${payment.months || 3} 个月的Premium会员`);
    } catch (error) {
      // 更新交易状态为失败
      db.updateTransactionStatus(payment.txHash, 'failed', {
        error: error.message,
        failedAt: new Date().toISOString()
      });
      
      log(`购买Premium会员失败: ${error.message}`, 'ERROR');
      
      // 这里可以添加通知管理员的逻辑
    } finally {
      // 关闭浏览器
      await splitBot.close();
    }
  } catch (error) {
    log(`处理支付失败: ${error.message}`, 'ERROR');
  }
}

/**
 * 启动集成服务
 */
async function startIntegrationService() {
  log('正在启动USDT转账监听与Split自动化购买集成服务...');
  
  try {
    // 初始化USDT监听器
    const monitor = new UsdtMonitor(config.usdt);
    
    // 设置支付成功回调
    monitor.onPaymentSuccess(handleNewPayment);
    
    // 启动监听
    await monitor.start();
    
    log('集成服务启动成功，正在监听USDT转账...');
  } catch (error) {
    log(`启动集成服务失败: ${error.message}`, 'ERROR');
    process.exit(1);
  }
}

// 启动服务
startIntegrationService();

// 处理进程退出
process.on('SIGINT', async () => {
  log('正在关闭服务...');
  process.exit(0);
});

module.exports = { startIntegrationService, handleNewPayment };
