/**
 * Split自动化脚本的错误处理与重试机制
 * 处理各种异常情况并实现智能重试
 */

const fs = require('fs');
const path = require('path');

class ErrorHandler {
  /**
   * 创建错误处理器实例
   * @param {Object} options 配置选项
   * @param {string} options.logPath 日志路径
   * @param {number} options.maxRetries 最大重试次数，默认3
   * @param {number} options.retryDelay 重试延迟(毫秒)，默认5000
   * @param {Function} options.notifyCallback 通知回调函数
   */
  constructor(options = {}) {
    this.options = {
      logPath: path.join(process.cwd(), 'logs'),
      maxRetries: 3,
      retryDelay: 5000,
      notifyCallback: null,
      ...options
    };

    this.errorLogFile = path.join(this.options.logPath, `error-${new Date().toISOString().replace(/:/g, '-')}.log`);
    
    // 确保日志目录存在
    if (!fs.existsSync(this.options.logPath)) {
      fs.mkdirSync(this.options.logPath, { recursive: true });
    }
  }

  /**
   * 记录错误日志
   * @param {Error} error 错误对象
   * @param {string} context 错误上下文
   */
  logError(error, context = '') {
    const timestamp = new Date().toISOString();
    const errorMessage = `[${timestamp}] [ERROR] ${context}: ${error.message}\n${error.stack}\n`;
    
    console.error(errorMessage);
    fs.appendFileSync(this.errorLogFile, errorMessage);
    
    // 如果提供了通知回调，则调用它
    if (typeof this.options.notifyCallback === 'function') {
      this.options.notifyCallback({
        timestamp,
        context,
        message: error.message,
        stack: error.stack
      });
    }
  }

  /**
   * 使用重试机制执行函数
   * @param {Function} fn 要执行的函数
   * @param {string} context 操作上下文
   * @param {Object} options 重试选项
   * @returns {Promise<any>} 函数执行结果
   */
  async withRetry(fn, context = '', options = {}) {
    const retryOptions = {
      maxRetries: this.options.maxRetries,
      retryDelay: this.options.retryDelay,
      ...options
    };
    
    let lastError;
    
    for (let attempt = 1; attempt <= retryOptions.maxRetries + 1; attempt++) {
      try {
        // 如果不是第一次尝试，记录重试信息
        if (attempt > 1) {
          console.log(`[RETRY] ${context}: 第 ${attempt - 1} 次重试...`);
        }
        
        // 执行函数
        return await fn();
      } catch (error) {
        lastError = error;
        
        // 记录错误
        this.logError(error, `${context} (尝试 ${attempt}/${retryOptions.maxRetries + 1})`);
        
        // 如果已达到最大重试次数，则抛出错误
        if (attempt > retryOptions.maxRetries) {
          throw new Error(`${context} 失败，已重试 ${retryOptions.maxRetries} 次: ${error.message}`);
        }
        
        // 等待一段时间后重试
        await new Promise(resolve => setTimeout(resolve, retryOptions.retryDelay));
      }
    }
  }

  /**
   * 检测并处理特定类型的错误
   * @param {Error} error 错误对象
   * @returns {Object} 错误类型和处理建议
   */
  analyzeError(error) {
    const errorMessage = error.message.toLowerCase();
    
    // 网络错误
    if (errorMessage.includes('net::') || errorMessage.includes('network') || errorMessage.includes('timeout')) {
      return {
        type: 'NETWORK_ERROR',
        suggestion: '检查网络连接，可能需要重试',
        retryable: true
      };
    }
    
    // 选择器错误（页面结构变化）
    if (errorMessage.includes('selector') || errorMessage.includes('xpath') || errorMessage.includes('element not found')) {
      return {
        type: 'SELECTOR_ERROR',
        suggestion: 'Split平台界面可能已更新，需要更新选择器',
        retryable: false
      };
    }
    
    // 钱包错误
    if (errorMessage.includes('wallet') || errorMessage.includes('ton') || errorMessage.includes('extension')) {
      return {
        type: 'WALLET_ERROR',
        suggestion: '检查TON钱包扩展是否正常工作，可能需要重新配置',
        retryable: false
      };
    }
    
    // 余额不足
    if (errorMessage.includes('balance') || errorMessage.includes('insufficient') || errorMessage.includes('not enough')) {
      return {
        type: 'INSUFFICIENT_BALANCE',
        suggestion: '钱包余额不足，需要充值',
        retryable: false
      };
    }
    
    // 默认为未知错误
    return {
      type: 'UNKNOWN_ERROR',
      suggestion: '未知错误，请检查日志获取详细信息',
      retryable: true
    };
  }
}

/**
 * Split自动化脚本的重试增强版
 * 扩展原有SplitAutomation类，添加错误处理和重试机制
 */
class SplitAutomationWithRetry {
  /**
   * 创建增强版Split自动化实例
   * @param {Object} splitOptions Split自动化选项
   * @param {Object} errorOptions 错误处理选项
   */
  constructor(splitOptions, errorOptions = {}) {
    // 导入原始SplitAutomation类
    const { SplitAutomation } = require('./split-automation');
    
    // 创建原始实例
    this.splitBot = new SplitAutomation(splitOptions);
    
    // 创建错误处理器
    this.errorHandler = new ErrorHandler(errorOptions);
    
    // 保存选项
    this.splitOptions = splitOptions;
    this.errorOptions = errorOptions;
  }

  /**
   * 初始化浏览器和TON钱包（带重试）
   */
  async initialize() {
    return this.errorHandler.withRetry(
      () => this.splitBot.initialize(),
      '初始化浏览器和TON钱包'
    );
  }

  /**
   * 连接TON钱包（带重试）
   */
  async connectWallet() {
    return this.errorHandler.withRetry(
      () => this.splitBot.connectWallet(),
      '连接TON钱包',
      { maxRetries: 5, retryDelay: 3000 } // 钱包连接可能需要更多重试
    );
  }

  /**
   * 为用户购买Premium会员（带重试）
   * @param {string} telegramId 用户的Telegram ID
   * @param {number} months 购买的月数
   */
  async buyPremium(telegramId, months = 3) {
    return this.errorHandler.withRetry(
      () => this.splitBot.buyPremium(telegramId, months),
      `为用户 ${telegramId} 购买 ${months} 个月的Premium会员`
    );
  }

  /**
   * 关闭浏览器（带重试）
   */
  async close() {
    try {
      await this.splitBot.close();
    } catch (error) {
      this.errorHandler.logError(error, '关闭浏览器');
      // 即使关闭失败也不抛出异常，避免影响主流程
    }
  }

  /**
   * 检查钱包余额是否足够
   * @param {number} requiredAmount 所需金额
   * @returns {Promise<boolean>} 余额是否足够
   */
  async checkWalletBalance(requiredAmount) {
    try {
      // 这里需要实现检查TON钱包余额的逻辑
      // 由于需要与具体钱包扩展交互，这里只提供框架
      
      // 示例实现
      const balance = await this.errorHandler.withRetry(
        async () => {
          // 获取钱包页面
          const pages = await this.splitBot.browser.pages();
          const walletPage = pages.find(p => p.url().includes('wallet'));
          
          if (!walletPage) {
            throw new Error('找不到钱包页面');
          }
          
          // 从钱包页面获取余额
          const balanceText = await walletPage.evaluate(() => {
            const balanceElement = document.querySelector('.balance');
            return balanceElement ? balanceElement.textContent : '0';
          });
          
          // 解析余额
          return parseFloat(balanceText.replace('TON', '').trim());
        },
        '检查钱包余额'
      );
      
      return balance >= requiredAmount;
    } catch (error) {
      this.errorHandler.logError(error, '检查钱包余额');
      // 如果无法检查余额，假设余额足够，让后续流程自行处理余额不足的情况
      return true;
    }
  }

  /**
   * 处理页面结构变化
   * 当检测到页面选择器失效时，尝试使用备用选择器
   * @param {string} action 操作名称
   * @returns {Promise<boolean>} 是否成功处理
   */
  async handlePageStructureChange(action) {
    try {
      // 根据不同操作使用备用选择器
      switch (action) {
        case 'connectWallet':
          // 尝试备用的钱包连接方法
          await this.splitBot.page.evaluate(() => {
            // 使用JavaScript直接点击元素
            const walletButtons = Array.from(document.querySelectorAll('button'))
              .filter(btn => btn.textContent.includes('Connect') || btn.textContent.includes('Wallet'));
            
            if (walletButtons.length > 0) {
              walletButtons[0].click();
            }
          });
          return true;
          
        case 'buyPremium':
          // 尝试备用的购买方法
          await this.splitBot.page.evaluate(() => {
            // 使用JavaScript直接点击元素
            const premiumButtons = Array.from(document.querySelectorAll('button'))
              .filter(btn => btn.textContent.includes('Premium') || btn.textContent.includes('Buy'));
            
            if (premiumButtons.length > 0) {
              premiumButtons[0].click();
            }
          });
          return true;
          
        default:
          return false;
      }
    } catch (error) {
      this.errorHandler.logError(error, `处理页面结构变化 (${action})`);
      return false;
    }
  }

  /**
   * 发送错误通知
   * @param {Object} error 错误信息
   */
  async sendErrorNotification(error) {
    try {
      // 这里实现发送通知的逻辑
      // 可以是发送邮件、短信、Telegram消息等
      
      console.log(`[NOTIFICATION] 发送错误通知: ${error.message}`);
      
      // 示例：写入通知日志
      const notificationLog = path.join(this.errorOptions.logPath || process.cwd(), 'notifications.log');
      fs.appendFileSync(notificationLog, `[${new Date().toISOString()}] ${JSON.stringify(error)}\n`);
      
      // 如果配置了通知回调，则调用它
      if (typeof this.errorOptions.notifyCallback === 'function') {
        await this.errorOptions.notifyCallback(error);
      }
    } catch (notifyError) {
      console.error(`发送通知失败: ${notifyError.message}`);
    }
  }
}

module.exports = { ErrorHandler, SplitAutomationWithRetry };
