/**
 * Split平台自动化脚本
 * 用于自动在Split平台上为用户购买Telegram Premium会员
 */

const puppeteer = require('puppeteer-extra');
const StealthPlugin = require('puppeteer-extra-plugin-stealth');
const fs = require('fs');
const path = require('path');

// 使用Stealth插件以避免被检测为自动化脚本
puppeteer.use(StealthPlugin());

class SplitAutomation {
  /**
   * 创建Split自动化实例
   * @param {Object} options 配置选项
   * @param {string} options.walletPath TON钱包扩展路径
   * @param {Object} options.credentials 凭证信息
   * @param {string} options.credentials.walletSeed 钱包种子短语
   * @param {string} options.credentials.password 钱包密码
   * @param {number} options.timeout 超时时间(毫秒)，默认30000
   * @param {boolean} options.headless 是否使用无头模式，默认false
   * @param {string} options.logPath 日志路径，默认为当前目录下的logs
   */
  constructor(options) {
    this.options = {
      timeout: 30000,
      headless: false,
      logPath: path.join(process.cwd(), 'logs'),
      ...options
    };

    // 确保必要的选项已提供
    if (!this.options.walletPath) {
      throw new Error('必须提供TON钱包扩展路径');
    }
    if (!this.options.credentials || !this.options.credentials.walletSeed) {
      throw new Error('必须提供钱包种子短语');
    }

    this.browser = null;
    this.page = null;
    this.isInitialized = false;
    this.logFile = path.join(this.options.logPath, `split-automation-${new Date().toISOString().replace(/:/g, '-')}.log`);

    // 确保日志目录存在
    if (!fs.existsSync(this.options.logPath)) {
      fs.mkdirSync(this.options.logPath, { recursive: true });
    }
  }

  /**
   * 记录日志
   * @param {string} message 日志消息
   * @param {string} level 日志级别
   */
  log(message, level = 'INFO') {
    const timestamp = new Date().toISOString();
    const logMessage = `[${timestamp}] [${level}] ${message}`;
    
    console.log(logMessage);
    fs.appendFileSync(this.logFile, logMessage + '\n');
  }

  /**
   * 初始化浏览器和TON钱包
   */
  async initialize() {
    try {
      this.log('正在初始化浏览器和TON钱包...');
      
      // 启动浏览器，加载TON钱包扩展
      this.browser = await puppeteer.launch({
        headless: this.options.headless,
        args: [
          `--disable-extensions-except=${this.options.walletPath}`,
          `--load-extension=${this.options.walletPath}`,
          '--no-sandbox',
          '--disable-setuid-sandbox'
        ],
        defaultViewport: null,
        timeout: this.options.timeout
      });

      // 等待TON钱包扩展加载
      const targets = await this.browser.targets();
      const extensionTarget = targets.find(target => 
        target.type() === 'service_worker' && 
        target.url().includes('background')
      );

      if (!extensionTarget) {
        throw new Error('无法加载TON钱包扩展');
      }

      // 打开新页面
      this.page = await this.browser.newPage();
      
      // 设置页面超时
      await this.page.setDefaultTimeout(this.options.timeout);
      
      // 导航到Split平台
      await this.page.goto('https://split.tg/premium', { 
        waitUntil: 'networkidle2' 
      });
      
      this.log('成功导航到Split平台');
      
      // 连接钱包
      await this.connectWallet();
      
      this.isInitialized = true;
      this.log('初始化完成');
      
      return true;
    } catch (error) {
      this.log(`初始化失败: ${error.message}`, 'ERROR');
      await this.close();
      throw error;
    }
  }

  /**
   * 连接TON钱包
   */
  async connectWallet() {
    try {
      this.log('正在连接TON钱包...');
      
      // 点击"Connect wallet"按钮
      await this.page.waitForSelector('button:contains("Connect wallet")');
      await this.page.click('button:contains("Connect wallet")');
      
      // 等待钱包连接弹窗
      await this.page.waitForSelector('#tc-widget-root');
      
      // 选择TON钱包
      await this.page.waitForSelector('button:contains("TON Wallet")');
      await this.page.click('button:contains("TON Wallet")');
      
      // 处理钱包授权
      // 注意：这部分可能需要根据实际TON钱包扩展的界面进行调整
      const pages = await this.browser.pages();
      const walletPage = pages.find(p => p.url().includes('wallet'));
      
      if (walletPage) {
        await walletPage.waitForSelector('button:contains("Connect")');
        await walletPage.click('button:contains("Connect")');
      }
      
      // 等待连接成功
      await this.page.waitForFunction(() => {
        const walletButton = document.querySelector('button:contains("Connect wallet")');
        return walletButton && walletButton.textContent !== 'Connect wallet';
      });
      
      this.log('TON钱包连接成功');
      return true;
    } catch (error) {
      this.log(`连接TON钱包失败: ${error.message}`, 'ERROR');
      throw error;
    }
  }

  /**
   * 为用户购买Premium会员
   * @param {string} telegramId 用户的Telegram ID
   * @param {number} months 购买的月数，可选值为1, 3, 6, 12
   * @returns {Promise<Object>} 购买结果
   */
  async buyPremium(telegramId, months = 3) {
    if (!this.isInitialized) {
      await this.initialize();
    }
    
    try {
      this.log(`开始为用户 ${telegramId} 购买 ${months} 个月的Premium会员...`);
      
      // 确保在Premium页面
      if (!this.page.url().includes('premium')) {
        await this.page.goto('https://split.tg/premium', { 
          waitUntil: 'networkidle2' 
        });
      }
      
      // 点击"Buy Premium to User"按钮
      await this.page.waitForSelector('button:contains("Buy Premium to User")');
      await this.page.click('button:contains("Buy Premium to User")');
      
      // 等待购买表单加载
      await this.page.waitForSelector('input[placeholder="Telegram ID or @username"]');
      
      // 输入Telegram ID
      await this.page.type('input[placeholder="Telegram ID or @username"]', telegramId);
      
      // 选择月数
      const validMonths = [1, 3, 6, 12];
      if (!validMonths.includes(months)) {
        months = 3; // 默认3个月
      }
      
      // 找到并点击对应月数的选项
      await this.page.waitForSelector(`button:contains("${months} month")`);
      await this.page.click(`button:contains("${months} month")`);
      
      // 点击确认购买按钮
      await this.page.waitForSelector('button:contains("Buy Premium")');
      await this.page.click('button:contains("Buy Premium")');
      
      // 等待支付确认弹窗
      await this.page.waitForSelector('button:contains("Confirm")');
      await this.page.click('button:contains("Confirm")');
      
      // 处理TON钱包支付确认
      // 注意：这部分可能需要根据实际TON钱包扩展的界面进行调整
      const pages = await this.browser.pages();
      const walletPage = pages.find(p => p.url().includes('wallet'));
      
      if (walletPage) {
        await walletPage.waitForSelector('button:contains("Confirm")');
        await walletPage.click('button:contains("Confirm")');
      }
      
      // 等待交易完成
      await this.page.waitForSelector('div:contains("Transaction completed")');
      
      // 获取交易ID或其他确认信息
      const transactionId = await this.page.evaluate(() => {
        const txElement = document.querySelector('div:contains("Transaction ID")');
        return txElement ? txElement.textContent.replace('Transaction ID:', '').trim() : 'unknown';
      });
      
      this.log(`成功为用户 ${telegramId} 购买了 ${months} 个月的Premium会员，交易ID: ${transactionId}`);
      
      return {
        success: true,
        telegramId,
        months,
        transactionId,
        timestamp: new Date().toISOString()
      };
    } catch (error) {
      this.log(`购买Premium会员失败: ${error.message}`, 'ERROR');
      throw error;
    }
  }

  /**
   * 关闭浏览器
   */
  async close() {
    if (this.browser) {
      try {
        await this.browser.close();
        this.log('浏览器已关闭');
      } catch (error) {
        this.log(`关闭浏览器失败: ${error.message}`, 'ERROR');
      }
      this.browser = null;
      this.page = null;
      this.isInitialized = false;
    }
  }
}

module.exports = { SplitAutomation };
