/**
 * Split自动化脚本测试工具
 * 用于测试和优化Split平台自动化购买流程
 */

const { SplitAutomationWithRetry } = require('./src/error-handler');
const fs = require('fs');
const path = require('path');

// 测试配置
const config = {
  // Split自动化配置
  split: {
    walletPath: process.env.TON_WALLET_PATH || '/path/to/ton/wallet/extension',
    credentials: {
      walletSeed: process.env.WALLET_SEED || 'your-wallet-seed-phrase',
      password: process.env.WALLET_PASSWORD || 'your-wallet-password'
    },
    headless: process.env.HEADLESS === 'true',
    timeout: parseInt(process.env.TIMEOUT || '60000'), // 测试时使用更长的超时时间
    logPath: path.join(__dirname, 'logs')
  },
  
  // 错误处理配置
  error: {
    logPath: path.join(__dirname, 'logs'),
    maxRetries: 5,
    retryDelay: 3000,
    notifyCallback: (error) => {
      console.log(`[NOTIFICATION] 测试中发生错误: ${error.message}`);
    }
  },
  
  // 测试配置
  test: {
    telegramId: process.env.TEST_TELEGRAM_ID || '123456789', // 测试用的Telegram ID
    months: parseInt(process.env.TEST_MONTHS || '1'), // 测试购买1个月
    resultPath: path.join(__dirname, 'test-results')
  }
};

// 确保结果目录存在
if (!fs.existsSync(config.test.resultPath)) {
  fs.mkdirSync(config.test.resultPath, { recursive: true });
}

/**
 * 记录测试结果
 * @param {string} testName 测试名称
 * @param {boolean} success 是否成功
 * @param {Object} details 详细信息
 */
function logTestResult(testName, success, details = {}) {
  const result = {
    testName,
    success,
    timestamp: new Date().toISOString(),
    details
  };
  
  const resultFile = path.join(config.test.resultPath, `${testName.replace(/\s+/g, '-')}.json`);
  fs.writeFileSync(resultFile, JSON.stringify(result, null, 2));
  
  console.log(`[TEST] ${testName}: ${success ? '成功' : '失败'}`);
  if (!success) {
    console.error(`[TEST] 失败详情: ${JSON.stringify(details)}`);
  }
}

/**
 * 测试初始化
 */
async function testInitialize() {
  const splitBot = new SplitAutomationWithRetry(config.split, config.error);
  
  try {
    await splitBot.initialize();
    logTestResult('初始化测试', true);
    return splitBot;
  } catch (error) {
    logTestResult('初始化测试', false, { error: error.message, stack: error.stack });
    throw error;
  } finally {
    await splitBot.close();
  }
}

/**
 * 测试钱包连接
 */
async function testWalletConnection() {
  const splitBot = new SplitAutomationWithRetry(config.split, config.error);
  
  try {
    await splitBot.initialize();
    await splitBot.connectWallet();
    logTestResult('钱包连接测试', true);
  } catch (error) {
    logTestResult('钱包连接测试', false, { error: error.message, stack: error.stack });
    throw error;
  } finally {
    await splitBot.close();
  }
}

/**
 * 测试Premium购买流程
 */
async function testBuyPremium() {
  const splitBot = new SplitAutomationWithRetry(config.split, config.error);
  
  try {
    await splitBot.initialize();
    const result = await splitBot.buyPremium(config.test.telegramId, config.test.months);
    logTestResult('Premium购买测试', true, { result });
  } catch (error) {
    logTestResult('Premium购买测试', false, { error: error.message, stack: error.stack });
    throw error;
  } finally {
    await splitBot.close();
  }
}

/**
 * 测试错误处理和重试机制
 */
async function testErrorHandling() {
  const splitBot = new SplitAutomationWithRetry(config.split, {
    ...config.error,
    maxRetries: 2, // 减少重试次数，加快测试
    retryDelay: 1000 // 减少重试延迟，加快测试
  });
  
  try {
    // 故意使用错误的选择器触发错误
    await splitBot.initialize();
    
    // 模拟选择器错误
    const originalBuyPremium = splitBot.splitBot.buyPremium;
    splitBot.splitBot.buyPremium = async () => {
      throw new Error('模拟的选择器错误: element not found');
    };
    
    try {
      await splitBot.buyPremium(config.test.telegramId, config.test.months);
      logTestResult('错误处理测试', false, { error: '应该抛出错误但没有' });
    } catch (error) {
      // 预期会抛出错误，这是正常的
      if (error.message.includes('已重试') && error.message.includes('模拟的选择器错误')) {
        logTestResult('错误处理测试', true, { error: error.message });
      } else {
        logTestResult('错误处理测试', false, { error: error.message, stack: error.stack });
        throw error;
      }
    } finally {
      // 恢复原始方法
      splitBot.splitBot.buyPremium = originalBuyPremium;
    }
  } catch (error) {
    logTestResult('错误处理测试', false, { error: error.message, stack: error.stack });
    throw error;
  } finally {
    await splitBot.close();
  }
}

/**
 * 测试页面结构变化处理
 */
async function testPageStructureChange() {
  const splitBot = new SplitAutomationWithRetry(config.split, config.error);
  
  try {
    await splitBot.initialize();
    
    // 测试处理页面结构变化的方法
    const result = await splitBot.handlePageStructureChange('connectWallet');
    
    logTestResult('页面结构变化处理测试', result, { result });
  } catch (error) {
    logTestResult('页面结构变化处理测试', false, { error: error.message, stack: error.stack });
    throw error;
  } finally {
    await splitBot.close();
  }
}

/**
 * 运行所有测试
 */
async function runAllTests() {
  console.log('[TEST] 开始运行Split自动化脚本测试...');
  
  try {
    // 按顺序运行测试
    await testInitialize();
    await testWalletConnection();
    await testErrorHandling();
    await testPageStructureChange();
    
    // 最后运行购买测试（可能会实际消费TON）
    if (process.env.RUN_PURCHASE_TEST === 'true') {
      await testBuyPremium();
    } else {
      console.log('[TEST] 跳过Premium购买测试（设置RUN_PURCHASE_TEST=true以启用）');
    }
    
    console.log('[TEST] 所有测试完成');
  } catch (error) {
    console.error(`[TEST] 测试过程中发生错误: ${error.message}`);
    process.exit(1);
  }
}

// 如果直接运行此文件，则执行测试
if (require.main === module) {
  runAllTests();
}

module.exports = { runAllTests, testInitialize, testWalletConnection, testBuyPremium, testErrorHandling, testPageStructureChange };
